<?php

declare(strict_types=1);

namespace Qls\ServicePoints\Test\Service;

use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Qls\ServicePoints\Service\CustomerAddressService;
use Qls\ServicePoints\Struct\ServicePoint;
use Shopware\Core\Checkout\Customer\Aggregate\CustomerAddress\CustomerAddressEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\EntitySearchResult;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;

class CustomerAddressServiceTest extends TestCase
{
    /** @var MockObject|Context */
    private MockObject $context;

    /** @var MockObject|EntityRepository */
    private MockObject $customerAddressRepository;

    private CustomerAddressService $customerAddressService;

    public function setUp(): void
    {
        $this->context = $this->getMockBuilder(Context::class)->disableOriginalConstructor()->getMock();
        $this->customerAddressRepository = $this->getMockBuilder(EntityRepository::class)->getMock();
        $this->customerAddressService = new CustomerAddressService(
            $this->customerAddressRepository
        );
    }

    public function testItAddsServicePointToCustomFields(): void
    {
        $customerAddressEntity = new CustomerAddressEntity();
        $customerAddressEntity->setId('::id::');

        $servicePoint = new ServicePoint(
            '::code::',
            '::name::',
            '::streetAndNumber::',
            '::postalCodeAndCity::',
            '::country::'
        );

        $this->customerAddressRepository
            ->expects($this->once())
            ->method('upsert')
            ->with([[
                'id' => '::id::',
                'customFields' => [
                    'service_point_code' => '::code::',
                    'service_point_name' => '::name::',
                    'service_point_street_and_number' => '::streetAndNumber::',
                    'service_point_postal_code_and_city' => '::postalCodeAndCity::',
                    'service_point_country' => '::country::'
                ]
            ]], $this->context);

        $this->customerAddressService->addServicePointToCustomFields($servicePoint, $customerAddressEntity, $this->context);
    }

    public function testItRemovesServicePointToCustomFields(): void
    {
        $customerAddressEntity = new CustomerAddressEntity();
        $customerAddressEntity->setId('::id::');

        $this->customerAddressRepository
            ->expects($this->once())
            ->method('upsert')
            ->with([[
                'id' => '::id::',
                'customFields' => null
            ]], $this->context);

        $this->customerAddressService->removeServicePointFromCustomFields($customerAddressEntity, $this->context);
    }

    public function testItEnsuresServicePointIsRemovedFromCustomFields(): void
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('id', '::id::'));

        $entitySearchResult = $this->getMockBuilder(EntitySearchResult::class)->disableOriginalConstructor()->getMock();

        $this->customerAddressRepository
            ->expects($this->once())
            ->method('search')
            ->with($criteria, $this->context)
            ->willReturn($entitySearchResult);

        $customerAddressEntity = new CustomerAddressEntity();
        $customerAddressEntity->setId('::id::');
        $customerAddressEntity->setCustomFields([
            'service_point_code' => '::code::',
        ]);

        $entitySearchResult
            ->expects($this->once())
            ->method('first')
            ->willReturn($customerAddressEntity);

        $this->customerAddressRepository
            ->expects($this->once())
            ->method('upsert')
            ->with([[
                'id' => '::id::',
                'customFields' => null
            ]], $this->context);

        $this->customerAddressService->ensureServicePointIsRemovedFromCustomFields('::id::', $this->context);
    }
}
