<?php

declare(strict_types=1);

namespace Qls\ServicePoints\Test\Service;

use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Qls\ServicePoints\Service\ShippingMethodService;
use Qls\ServicePoints\Service\SystemConfigService;
use Qls\ServicePoints\Struct\ServicePointInformation;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\Checkout\Cart\Delivery\Struct\Delivery;
use Shopware\Core\Checkout\Cart\Delivery\Struct\DeliveryCollection;
use Shopware\Core\Checkout\Cart\Delivery\Struct\ShippingLocation;
use Shopware\Core\Checkout\Customer\Aggregate\CustomerAddress\CustomerAddressEntity;
use Shopware\Core\Checkout\Shipping\ShippingMethodCollection;
use Shopware\Core\Checkout\Shipping\ShippingMethodEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenContainerEvent;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenEvent;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\EntitySearchResult;
use Shopware\Core\System\Country\CountryEntity;
use Shopware\Core\System\SystemConfig\SystemConfigEntity;

class ShippingMethodServiceTest extends TestCase
{
    /** @var MockObject|Context */
    private MockObject $context;

    /** @var MockObject|EntityRepository */
    private MockObject $ruleRepository;

    /** @var MockObject|EntityRepository */
    private MockObject $deliveryTimeRepository;

    /** @var MockObject|EntityRepository */
    private MockObject $shippingMethodRepository;

    /** @var MockObject|SystemConfigService */
    private MockObject $systemConfigService;

    private ShippingMethodService $shippingMethodService;

    public function setUp(): void
    {
        $this->context = $this->getMockBuilder(Context::class)->disableOriginalConstructor()->getMock();
        $this->ruleRepository = $this->getMockBuilder(EntityRepository::class)->getMock();
        $this->deliveryTimeRepository = $this->getMockBuilder(EntityRepository::class)->getMock();
        $this->shippingMethodRepository = $this->getMockBuilder(EntityRepository::class)->getMock();
        $this->systemConfigService = $this->getMockBuilder(SystemConfigService::class)->disableOriginalConstructor()->getMock();
        $this->shippingMethodService = new ShippingMethodService(
            $this->ruleRepository,
            $this->deliveryTimeRepository,
            $this->shippingMethodRepository,
            $this->systemConfigService,
        );
    }

    public function testItCreatesServicePointShippingMethod(): void
    {
        $entitySearchResult = $this->getMockBuilder(EntitySearchResult::class)
            ->disableOriginalConstructor()
            ->getMock();

        $rule = new SystemConfigEntity();
        $rule->setId('::rule_id::');

        $this->ruleRepository
            ->expects($this->once())
            ->method('search')
            ->with(new Criteria())
            ->willReturn($entitySearchResult);

        $entitySearchResult
            ->expects($this->once())
            ->method('first')
            ->willReturn($rule);

        $entitySearchResult = $this->getMockBuilder(EntitySearchResult::class)
            ->disableOriginalConstructor()
            ->getMock();

        $deliveryTime = new SystemConfigEntity();
        $deliveryTime->setId('::delivery_time_id::');

        $this->deliveryTimeRepository
            ->expects($this->once())
            ->method('search')
            ->with(new Criteria())
            ->willReturn($entitySearchResult);

        $entitySearchResult
            ->expects($this->once())
            ->method('first')
            ->willReturn($deliveryTime);

        $entityWrittenContainerEvent = $this->getMockBuilder(EntityWrittenContainerEvent::class)
            ->disableOriginalConstructor()
            ->getMock();

        $this->shippingMethodRepository
            ->expects($this->once())
            ->method('create')
            ->with([[
                'translations' => [
                    'en-GB' => [
                        'name' => 'Service Point Delivery',
                    ],
                    'de-DE' => [
                        'name' => 'Service Point Zustellung',
                    ],
                    'nl-NL' => [
                        'name' => 'Service Punt Levering',
                    ],
                    'es-ES' => [
                        'name' => 'Punto de servicio de entrega',
                    ],
                    'fr-FR' => [
                        'name' => 'Livraison en Point Relais',
                    ]
                ],
                'active' => false,
                'availabilityRuleId' => '::rule_id::',
                'deliveryTimeId' => '::delivery_time_id::',
            ]], $this->context)
            ->willReturn($entityWrittenContainerEvent);

        $entityWrittenEvent = $this->getMockBuilder(EntityWrittenEvent::class)
            ->disableOriginalConstructor()
            ->getMock();

        $entityWrittenContainerEvent
            ->expects($this->once())
            ->method('getEventByEntityName')
            ->with('shipping_method')
            ->willReturn($entityWrittenEvent);

        $entityWrittenEvent
            ->expects($this->once())
            ->method('getIds')
            ->willReturn(['::id::']);

        $this->systemConfigService
            ->expects($this->once())
            ->method('storeServicePointSystemConfig')
            ->with('::id::', $this->context);

        $this->shippingMethodService->createServicePointShippingMethod($this->context);
    }

    public function testItDoesntAddExtensionsToNonServicePointShippingMethod(): void
    {
        $shippingMethodEntity = $this->getMockBuilder(ShippingMethodEntity::class)->getMock();
        $cart = $this->getMockBuilder(Cart::class)->disableOriginalConstructor()->getMock();
        $shippingMethodCollection = new ShippingMethodCollection([
            $shippingMethodEntity,
        ]);

        $systemConfigEntity = new SystemConfigEntity();
        $systemConfigEntity->setConfigurationValue('::configuration_value::');

        $this->systemConfigService
            ->expects($this->once())
            ->method('getServicePointSystemConfig')
            ->with($this->context)
            ->willReturn($systemConfigEntity);

        $shippingMethodEntity
            ->expects($this->once())
            ->method('getId')
            ->willReturn('::id::');

        $shippingMethodEntity
            ->expects($this->never())
            ->method('addExtension');

        $this->shippingMethodService->addExtensionsToServicePointShippingMethod($shippingMethodCollection, $cart, $this->context);
    }

    public function testItAddsDefaultExtensionsToServicePointShippingMethod(): void
    {
        $shippingMethodEntity = $this->getMockBuilder(ShippingMethodEntity::class)->getMock();
        $cart = $this->getMockBuilder(Cart::class)->disableOriginalConstructor()->getMock();
        $shippingMethodCollection = new ShippingMethodCollection([
            $shippingMethodEntity,
        ]);

        $systemConfigEntity = new SystemConfigEntity();
        $systemConfigEntity->setConfigurationValue('::configuration_value::');

        $this->systemConfigService
            ->expects($this->once())
            ->method('getServicePointSystemConfig')
            ->with($this->context)
            ->willReturn($systemConfigEntity);

        $companyIdConfigEntity = new SystemConfigEntity();
        $companyIdConfigEntity->setConfigurationValue('::companyId::');

        $this->systemConfigService
            ->expects($this->once())
            ->method('getCompanyIdConfig')
            ->with($this->context)
            ->willReturn($companyIdConfigEntity);

        $shippingMethodEntity
            ->expects($this->once())
            ->method('getId')
            ->willReturn('::configuration_value::');

        $shippingMethodEntity
            ->expects($this->once())
            ->method('addExtension')
            ->with('servicePointInformation', new ServicePointInformation(
                true,
                'https://embed.pakketdienstqls.nl/servicepoint/locator/NL/?companyId=::companyId::#postalcode=3313LC',
                null
            ));

        $this->shippingMethodService->addExtensionsToServicePointShippingMethod($shippingMethodCollection, $cart, $this->context);
    }

    public function testItAddsClosestExtensionsToServicePointShippingMethod(): void
    {
        $shippingMethodEntity = $this->getMockBuilder(ShippingMethodEntity::class)->getMock();
        $cart = $this->getMockBuilder(Cart::class)->disableOriginalConstructor()->getMock();
        $shippingMethodCollection = new ShippingMethodCollection([
            $shippingMethodEntity,
        ]);

        $systemConfigEntity = new SystemConfigEntity();
        $systemConfigEntity->setConfigurationValue('::configuration_value::');

        $this->systemConfigService
            ->expects($this->once())
            ->method('getServicePointSystemConfig')
            ->with($this->context)
            ->willReturn($systemConfigEntity);

        $this->systemConfigService
            ->expects($this->once())
            ->method('getCompanyIdConfig')
            ->with($this->context)
            ->willReturn(null);

        $shippingMethodEntity
            ->expects($this->once())
            ->method('getId')
            ->willReturn('::configuration_value::');

        $delivery = $this->getMockBuilder(Delivery::class)
            ->disableOriginalConstructor()
            ->getMock();

        $cart
            ->expects($this->once())
            ->method('getDeliveries')
            ->willReturn(new DeliveryCollection([
                $delivery
            ]));

        $country = new CountryEntity();
        $country->setIso('::iso::');

        $customerAddress = new CustomerAddressEntity();
        $customerAddress->setZipcode('::zip_code::');
        $customerAddress->setCountry($country);

        $delivery
            ->expects($this->once())
            ->method('getLocation')
            ->willReturn(new ShippingLocation(
                $country,
                null,
                $customerAddress
            ));

        $shippingMethodEntity
            ->expects($this->once())
            ->method('addExtension')
            ->with('servicePointInformation', new ServicePointInformation(
                true,
                'https://embed.pakketdienstqls.nl/servicepoint/locator/::iso::/?companyId=#postalcode=::zip_code::',
                null
            ));

        $this->shippingMethodService->addExtensionsToServicePointShippingMethod($shippingMethodCollection, $cart, $this->context);
    }
}
