<?php

declare(strict_types=1);

namespace Qls\ServicePoints\Test\Service;

use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Qls\ServicePoints\Service\SystemConfigService;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\EntitySearchResult;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\SystemConfig\SystemConfigEntity;

class SystemConfigServiceTest extends TestCase
{
    /** @var MockObject|Context */
    private MockObject $context;

    /** @var MockObject|EntityRepository */
    private MockObject $systemConfigRepository;

    private SystemConfigService $systemConfigService;

    public function setUp(): void
    {
        $this->context = $this->getMockBuilder(Context::class)->disableOriginalConstructor()->getMock();
        $this->systemConfigRepository = $this->getMockBuilder(EntityRepository::class)->getMock();
        $this->systemConfigService = new SystemConfigService(
            $this->systemConfigRepository
        );
    }

    public function testItStoresServicePointSystemConfig(): void
    {
        $this->systemConfigRepository
            ->expects($this->once())
            ->method('create')
            ->with([[
                'configurationKey' => 'qlsServicePoints.servicePointShippingMethodId',
                'configurationValue' => '::shipping_method_id::',
            ]], $this->context);

        $this->systemConfigService->storeServicePointSystemConfig('::shipping_method_id::', $this->context);
    }

    public function testItGetsServicePointSystemConfig(): void
    {
        $entitySearchResult = $this->getMockBuilder(EntitySearchResult::class)
            ->disableOriginalConstructor()
            ->getMock();

        $criteria = (new Criteria())->addFilter(
            new EqualsFilter('configurationKey', 'qlsServicePoints.servicePointShippingMethodId')
        );

        $this->systemConfigRepository
            ->expects($this->once())
            ->method('search')
            ->with($criteria)
            ->willReturn($entitySearchResult);

        $entitySearchResult
            ->expects($this->once())
            ->method('first')
            ->willReturn(new SystemConfigEntity());

        $this->systemConfigService->getServicePointSystemConfig($this->context);
    }

    public function testItEnsuresServicePointSystemConfigIsDeletedWhenItExists(): void
    {
        $entitySearchResult = $this->getMockBuilder(EntitySearchResult::class)
            ->disableOriginalConstructor()
            ->getMock();

        $criteria = (new Criteria())->addFilter(
            new EqualsFilter('configurationKey', 'qlsServicePoints.servicePointShippingMethodId')
        );

        $this->systemConfigRepository
            ->expects($this->once())
            ->method('search')
            ->with($criteria)
            ->willReturn($entitySearchResult);

        $systemConfigEntity = new SystemConfigEntity();
        $systemConfigEntity->setId('::id::');

        $entitySearchResult
            ->expects($this->once())
            ->method('first')
            ->willReturn($systemConfigEntity);

        $this->systemConfigRepository
            ->expects($this->once())
            ->method('delete')
            ->with([[
                'id' => '::id::',
            ]], $this->context);

        $this->systemConfigService->ensureServicePointSystemConfigIsDeleted($this->context);
    }

    public function testItEnsuresServicePointSystemConfigIsDeletedWhenItDoesntExist(): void
    {
        $entitySearchResult = $this->getMockBuilder(EntitySearchResult::class)
            ->disableOriginalConstructor()
            ->getMock();

        $criteria = (new Criteria())->addFilter(
            new EqualsFilter('configurationKey', 'qlsServicePoints.servicePointShippingMethodId')
        );

        $this->systemConfigRepository
            ->expects($this->once())
            ->method('search')
            ->with($criteria)
            ->willReturn($entitySearchResult);

        $entitySearchResult
            ->expects($this->once())
            ->method('first')
            ->willReturn(null);

        $this->systemConfigService->ensureServicePointSystemConfigIsDeleted($this->context);
    }
}
